@@ -6,10 +6,26 @@ import { ConnectionTimeoutError, ClientClosedError, SocketClosedUnexpectedlyErro
 import { promiseTimeout } from '../utils';
 
 export interface RedisSocketCommonOptions {
+    /**
+     * Connection Timeout (in milliseconds)
+     */
     connectTimeout?: number;
+    /**
+     * Toggle [`Nagle's algorithm`](https://nodejs.org/api/net.html#net_socket_setnodelay_nodelay)
+     */
     noDelay?: boolean;
+    /**
+     * Toggle [`keep-alive`](https://nodejs.org/api/net.html#net_socket_setkeepalive_enable_initialdelay)
+     */
     keepAlive?: number | false;
-    reconnectStrategy?(retries: number): number | Error;
+    /**
+     * When the socket closes unexpectedly (without calling `.quit()`/`.disconnect()`), the client uses `reconnectStrategy` to decide what to do. The following values are supported:
+     * 1. `false` -> do not reconnect, close the client and flush the command queue.
+     * 2. `number` -> wait for `X` milliseconds before reconnecting.
+     * 3. `(retries: number, cause: Error) => false | number | Error` -> `number` is the same as configuring a `number` directly, `Error` is the same as `false`, but with a custom error.
+     * Defaults to `retries => Math.min(retries * 50, 500)`
+     */
+    reconnectStrategy?: false | number | ((retries: number, cause: Error) => false | Error | number);
 }
 
 type RedisNetSocketOptions = Partial<net.SocketConnectOpts> & {
@@ -83,23 +99,42 @@ export default class RedisSocket extends EventEmitter {
         this.#options = RedisSocket.#initiateOptions(options);
     }
 
-    reconnectStrategy(retries: number): number | Error {
-        if (this.#options.reconnectStrategy) {
+    #reconnectStrategy(retries: number, cause: Error) {
+        if (this.#options.reconnectStrategy === false) {
+            return false;
+        } else if (typeof this.#options.reconnectStrategy === 'number') {
+            return this.#options.reconnectStrategy;
+        } else if (this.#options.reconnectStrategy) {
             try {
-                const retryIn = this.#options.reconnectStrategy(retries);
-                if (typeof retryIn !== 'number' && !(retryIn instanceof Error)) {
-                    throw new TypeError('Reconnect strategy should return `number | Error`');
+                const retryIn = this.#options.reconnectStrategy(retries, cause);
+                if (retryIn !== false && !(retryIn instanceof Error) && typeof retryIn !== 'number') {
+                    throw new TypeError(`Reconnect strategy should return \`false | Error | number\`, got ${retryIn} instead`);
                 }
 
                 return retryIn;
             } catch (err) {
-                this.emit('error', err);
+                this.emit('error', err);    
             }
         }
 
         return Math.min(retries * 50, 500);
     }
 
+    #shouldReconnect(retries: number, cause: Error) {
+        const retryIn = this.#reconnectStrategy(retries, cause);
+        if (retryIn === false) {
+            this.#isOpen = false;
+            this.emit('error', cause);
+            return cause;
+        } else if (retryIn instanceof Error) {
+            this.#isOpen = false;
+            this.emit('error', cause);
+            return new ReconnectStrategyError(retryIn, cause);
+        }
+
+        return retryIn;
+    }
+
     async connect(): Promise<void> {
         if (this.#isOpen) {
             throw new Error('Socket already opened');
@@ -109,13 +144,9 @@ export default class RedisSocket extends EventEmitter {
         return this.#connect();
     }
 
-    async #connect(hadError?: boolean): Promise<void> {
+    async #connect(): Promise<void> {
         let retries = 0;
         do {
-            if (retries > 0 || hadError) {
-                this.emit('reconnecting');
-            }
-
             try {
                 this.#socket = await this.#createSocket();
                 this.#writableNeedDrain = false;
@@ -131,17 +162,15 @@ export default class RedisSocket extends EventEmitter {
                 this.#isReady = true;
                 this.emit('ready');
             } catch (err) {
-                const retryIn = this.reconnectStrategy(retries);
-                if (retryIn instanceof Error) {
-                    this.#isOpen = false;
-                    this.emit('error', err);
-                    throw new ReconnectStrategyError(retryIn, err);
+                const retryIn = this.#shouldReconnect(retries++, err as Error);
+                if (typeof retryIn !== 'number') {
+                    throw retryIn;
                 }
 
                 this.emit('error', err);
                 await promiseTimeout(retryIn);
+                this.emit('reconnecting');
             }
-            retries++;
         } while (this.#isOpen && !this.#isReady);
     }
 
@@ -200,12 +229,14 @@ export default class RedisSocket extends EventEmitter {
     }
 
     #onSocketError(err: Error): void {
+        const wasReady = this.#isReady;
         this.#isReady = false;
         this.emit('error', err);
 
-        if (!this.#isOpen) return;
-
-        this.#connect(true).catch(() => {
+        if (!wasReady || !this.#isOpen || typeof this.#shouldReconnect(0, err) !== 'number') return;
+        
+        this.emit('reconnecting');
+        this.#connect().catch(() => {
             // the error was already emitted, silently ignore it
         });
     }
@@ -240,14 +271,15 @@ export default class RedisSocket extends EventEmitter {
         this.emit('end');
     }
 
-    async quit(fn: () => Promise<unknown>): Promise<void> {
+    async quit<T>(fn: () => Promise<T>): Promise<T> {
         if (!this.#isOpen) {
             throw new ClientClosedError();
         }
 
         this.#isOpen = false;
-        await fn();
+        const reply = await fn();
         this.#disconnect();
+        return reply;
     }
 
     #isCorked = false;
@@ -260,7 +292,7 @@ export default class RedisSocket extends EventEmitter {
         this.#socket.cork();
         this.#isCorked = true;
 
-        queueMicrotask(() => {
+        setImmediate(() => {
             this.#socket?.uncork();
             this.#isCorked = false;
         });
