@@ -108,6 +108,7 @@ export function transformCommandArguments<T = ClientCommandOptions>(
     command: RedisCommand,
     args: Array<unknown>
 ): {
+    jsArgs: Array<unknown>;
     args: RedisCommandArguments;
     options: CommandOptions<T> | undefined;
 } {
@@ -118,6 +119,7 @@ export function transformCommandArguments<T = ClientCommandOptions>(
     }
 
     return {
+        jsArgs: args,
         args: command.transformArguments(...args),
         options
     };
