@@ -15,12 +15,14 @@ type XInfoConsumersReply = Array<{
     name: RedisCommandArgument;
     pending: number;
     idle: number;
+    inactive: number;
 }>;
 
 export function transformReply(rawReply: Array<any>): XInfoConsumersReply {
     return rawReply.map(consumer => ({
         name: consumer[1],
         pending: consumer[3],
-        idle: consumer[5]
+        idle: consumer[5],
+        inactive: consumer[7]
     }));
 }
