@@ -20,6 +20,7 @@ import * as INFO from './INFO';
 import * as PROFILESEARCH from './PROFILE_SEARCH';
 import * as PROFILEAGGREGATE from './PROFILE_AGGREGATE';
 import * as SEARCH from './SEARCH';
+import * as SEARCH_NOCONTENT from './SEARCH_NOCONTENT';
 import * as SPELLCHECK from './SPELLCHECK';
 import * as SUGADD from './SUGADD';
 import * as SUGDEL from './SUGDEL';
@@ -80,6 +81,8 @@ export default {
     profileAggregate: PROFILEAGGREGATE,
     SEARCH,
     search: SEARCH,
+    SEARCH_NOCONTENT,
+    searchNoContent: SEARCH_NOCONTENT,
     SPELLCHECK,
     spellCheck: SPELLCHECK,
     SUGADD,
@@ -134,8 +137,8 @@ export enum RedisSearchLanguages {
 
 export type PropertyName = `${'@' | '$.'}${string}`;
 
-export type SortByProperty = PropertyName | {
-    BY: PropertyName;
+export type SortByProperty = string | {
+    BY: string;
     DIRECTION?: 'ASC' | 'DESC';
 };
 
@@ -187,7 +190,7 @@ export enum SchemaFieldTypes {
 
 type CreateSchemaField<
     T extends SchemaFieldTypes,
-    E = Record<keyof any, any>
+    E = Record<PropertyKey, unknown>
 > = T | ({
     type: T;
     AS?: string;
@@ -195,7 +198,7 @@ type CreateSchemaField<
 
 type CreateSchemaCommonField<
     T extends SchemaFieldTypes,
-    E = Record<string, never>
+    E = Record<PropertyKey, unknown>
 > = CreateSchemaField<
     T,
     ({
@@ -506,6 +509,14 @@ export function pushSearchOptions(
         args.push('DIALECT', options.DIALECT.toString());
     }
 
+    if (options?.RETURN?.length === 0) {
+        args.preserve = true;
+    }
+
+    if (options?.TIMEOUT !== undefined) {
+        args.push('TIMEOUT', options.TIMEOUT.toString());
+    }
+
     return args;
 }
 
